"""Py3o methods
"""
import atexit
import logging

from docker.errors import APIError  # type: ignore[import]

from .docker_client import DockerClient

_logger = logging.getLogger(__name__)


# TODO move to docker_client directly
def find_container(name):
    """Return container object from its name"""
    for container in DockerClient.client.containers():
        if name == container.names:
            return container
    return None


def remove_and_stop(docker_client, name):
    # TODO handle the case when the container has been created, not running
    tostop = find_container(name)
    docker_client.stop(tostop.get("Id"))
    docker_client.remove_container(tostop.get("Id"))


def start_py3o(host_fusion_port=8765, stop_at_exit=True):
    """start a py3o container"""
    docker_client = DockerClient.client
    fusion_repository = "xcgd/py3o"
    fusion_version = "1.0.0"
    fusion_name = "py3o_fusion"
    fusion_image = "{}:{}".format(fusion_repository, fusion_version)
    try:
        _logger.debug("Pulling %s", fusion_repository)
        docker_client.images.pull(repository=fusion_repository, tag=fusion_version)
    except APIError as e:
        _logger.warning("Exception when trying to pull: %s", e)
    if any(
        fusion_name == container.name for container in docker_client.containers.list()
    ):
        _logger.debug("%s Container already running", fusion_name)
        remove_and_stop(docker_client, fusion_name)
    # TODO handle --host-network option
    port_bindings = {8765: host_fusion_port}
    _logger.debug("Starting %s container", fusion_name)
    fusion = docker_client.containers.create(
        image=fusion_image, ports=port_bindings, name=fusion_name
    )
    _logger.debug("Starting %s container", fusion_name)
    fusion.start()

    def stop_py3o():
        # TODO test if still exists (no error if already stopped)
        _logger.info("Stopping fusion container")
        fusion.stop()
        fusion.wait()
        _logger.info("Removing fusion container")
        fusion.remove()

    if stop_at_exit:
        atexit.register(stop_py3o)
    return fusion_name, stop_py3o
